/*
 A simple program to generate random points in a hyperspheric uniform distribution
 Copyright (C) 2003, Johann Dro

 This program is free software; you can redistribute it and/or
 modify it under the terms of the GNU General Public License
 as published by the Free Software Foundation; either version 2
 of the License, or (at your option) any later version.

 This program is distributed in the hope that it will be useful,
 but WITHOUT ANY WARRANTY; without even the implied warranty of
 MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 GNU General Public License for more details.

 You should have received a copy of the GNU General Public License
 along with this program; if not, write to the Free Software
 Foundation, Inc., 59 Temple Place - Suite 330, Boston, MA  02111-1307, USA.
*/


#include <iostream>
#include <cstdlib>
#include <cmath>
#include <vector>
#include <string>
#include <ctime>

using namespace std;


/* Imprime le contenu d'un vecteur avec un sparateur */
void printVector( vector<double> aVector, string aSeparator, ostream * outer )
{
  *outer << aVector[0];
  
  for(unsigned int i=1; i<aVector.size(); i++ ) {
    *outer << aSeparator << aVector[i];
  }
}


/* Tirage alatoire d'un nombre dans une loi uniforme */
double randomUniform(double min, double max)
{
  double r;

  // r dans [0,1]
  r=(double)rand()/RAND_MAX;
  // r dans [min, max]
  r=min+r*(max-min);

  return r;
}



/********** Dbut bonne mthode  **********/

/* Tirage alatoire d'un nombre dans une loi normale */
double randomNormal(double mean, double std)
{
  double x1, x2, w, y1;

  do {
    x1 = 2.0 * randomUniform(0,1) - 1.0;
    x2 = 2.0 * randomUniform(0,1) - 1.0;
    w = x1 * x1 + x2 * x2;
  } while ( w >= 1.0);

  w = sqrt( (-2.0 * log( w ) ) / w );
  y1 = x1 * w;
  y1 = (y1 * std) + mean;

  return y1;
}

/* Tirage alatoire d'un vecteur hyper-sphrique selon une loi uniforme */
vector<double> randomHyperSphere(vector<double> center, double radius=1)
{
  // nb de dimensions tir de la taille du vecteur
  unsigned int N = center.size();
  
  // rsultat
  vector<double> x;
  x.reserve(N);
  double l;

  // Formule en LateX : 
  // $x_{i}=\frac{r^{\frac{1}{N}}_i\cdot a_{i}}{\sqrt{{\displaystyle \sum _{j=1}^{N}a_{i}}}}$

  for(unsigned int i=0; i<N; i++) {
    x.push_back( randomNormal(0,1) );
    l += x[i]*x[i];
  }
  l = sqrt(l);
  
  double r = randomUniform(0,1);
  r = pow( r, 1/(double)N);
  for(unsigned int i=0; i<N; i++) {
    x[i] = center[i] + radius*(r * x[i] / l);
  }
  
  return x;
}

/********** Fin bonne mthode  **********/



/********** Dbut mauvaise mthode  **********/

/* transforme des coordonnes (2D) polaires en coordonnes rectangulaires */
vector<double> Pol2Rec_2D(vector<double> PolVec)
{// r, a
  vector<double> RecVec;
  RecVec.reserve(2);

  RecVec.push_back( PolVec[0]*cos(PolVec[1]) ); // x
  RecVec.push_back( PolVec[0]*sin(PolVec[1]) ); // y

  return RecVec;
}


/* Tirage alatoire d'un vecteur hyper-sphrique selon une loi uniforme
   cette mthode est la mauvaise mthode, en 2D uniquement pour l'exemple */
vector<double> randomFalseHyperSphere(vector<double> center, double radius=1) 
{

  // nb de dimensions tir de la taille du vecteur
  unsigned int N = center.size();
  
  // rsultat
  vector<double> x;
  x.reserve(N);
  
  
  double Pi=3.141592653589793238462;

  // un rayon au hasard
  double r = randomUniform(0,radius);
  x.push_back(r);

  for(unsigned int i=1; i<N; i++)
    {
      r = randomUniform(0,2*Pi);  // un angle au hasard sur 2Pi
      x.push_back(r);
    }

  x = Pol2Rec_2D(x);
  
  x[0] += center[0];
  x[1] += center[1];
  
  return x;

}

/********** Fin mauvaise mthode  **********/



/* interface */
int main(unsigned  int argc, char* argv[] ) {
  
  int dim = 1;
  unsigned int  nbPoints = 1;
  double radius = 1;
  int isBad = 0;
  
  // boucle sur tout les arguments
  for(unsigned int i=1; i<argc; i++) {

    // dimension
    if(string(argv[i]) == "-d" || string(argv[i]) == "--dimension") {
      dim =  atoi(argv[i+1] );
      i++; continue;
    }
    
    // nombre de points
    if(string(argv[i]) == "-n" || string(argv[i]) == "--number") {
      nbPoints =  atoi(argv[i+1] );
      i++; continue;
    }
    
    // rayon de la sphre
    if(string(argv[i]) == "-r" || string(argv[i]) == "--radius") {
      radius =  atof(argv[i+1] );
      i++; continue;
    }
    
    // test avec la mauvaise mthode
    if(string(argv[i]) == "-f" || string(argv[i]) == "--false") {
      isBad =  1;
      continue;
    }
    
    // aide
    if(string(argv[i]) == "-h" || string(argv[i]) == "--help") {
      cout << "Usage: randomHyperSphere [OPTIONS]..." << endl;
      cout << "Effectue des tirages alatoires dans une hyper-sphre" << endl << endl;
      cout << "Les arguments obligatoires pour les options de formes longues le sont aussi pour les options de formes courtes." << endl;
      cout << "OPTIONS :" << endl;
      cout << "-d\t--dimension=DIMENSION\tnombre de variables" << endl;
      cout << "-n\t--number=POINTS\t\tnombre de points" << endl;
      cout << "-r\t--radius=RAYON\t\trayon de la sphre" << endl;
      cout << "-f\t--false\t\t\tutiliser la mthode biaise" << endl;
      return 0;
    }
    
    // version
    if(string(argv[i]) == "-v" || string(argv[i]) == "--version") {
      cout << "randomHyperSphere 1.0" << endl;
      return 0;
    }
    
  }
  
  
  // initialisation gnrateur alatoire
  time_t t;
  t = time(&t);
  srand(t);

  if(isBad==0) { // Bonne mthode

    // calculs
    for( unsigned int i=0; i<nbPoints; i++) {
      srand(t+i);
      // centre
      vector<double> c(dim,0);
      vector<double> p = randomHyperSphere(c,radius);
    
      printVector(p," ", &cout );
      cout << endl;
    }

  } else { // Mauvaise mthode
    
    // calculs
    for( unsigned int i=0; i<nbPoints; i++) {
      srand(t+i);
      // centre
      vector<double> c(dim,0);
      vector<double> p = randomFalseHyperSphere(c,radius);
    
      printVector(p," ", &cout );
      cout << endl;
    }

  }

}
